//***************************************************************************
//  interrupts.c
//***************************************************************************


unsigned char	ADCresults[2];	// Holds ADC measurements for LineR & LineL
unsigned char	lineChnls[] = {0, 1};	// ANx channels to be converted
unsigned char	chnlIndex;		// Index into lineChnls[]

extern unsigned char 	leftMotorTargetSpeed, rightMotorTargetSpeed;	// Target speeds 
extern unsigned char 	leftMotorAccel, rightMotorAccel;				// Allowed accelerations


//***************************************************************************
//	Tell compiler where the low priority interrupt vector is located.
//***************************************************************************
//#pragma code low_vector_section=0x18	// #pragma code low_vector=0x18
#pragma code low_vector_section=0x1018	// #pragma code low_vector=0x18
void low_vector (void){					// void low_interrupt (void)
  _asm GOTO LowPriorityTable _endasm
}
#pragma code

//***************************************************************************
//	Tell compiler where the high priority interrupt vector is located.
//***************************************************************************
//#pragma code high_vector_section=0x8	// #pragma code high_vector=0x08
#pragma code high_vector_section=0x1008	// #pragma code high_vector=0x08
void high_vector (void){					// void high_interrupt (void)
  _asm GOTO HighPriorityTable _endasm
}
#pragma code

//***************************************************************************
//	Low priority interrupts
//		Timer0 (global clock)
//***************************************************************************
#pragma interruptlow LowPriorityTable
void LowPriorityTable(void){
	static unsigned int heartbeat=HEARTBEAT_RATE, alarm=ALARM_RATE;
		//static unsigned char lcd=5; // update LCD every 5ms from the buffer
	unsigned char speed;
	if (INTCONbits.TMR0IF == 1) {		
		INTCONbits.TMR0IF = 0;			// Clear Timer0 interrupt flag
		TMR0H = 0xE8;					// Reset the timer
		TMR0L = 0x90;
		systemTime++;					// Increment systemTime

//	Adjust motors.
		if (flags.motorRampEnabled) {	// Ramp the motor speeds?
			if (MOTOR_LF != leftMotorTargetSpeed) {		// Adjust left motor if necessary
				if (MOTOR_LF < leftMotorTargetSpeed) {
					speed = MOTOR_LF + leftMotorAccel;
					if (speed <= leftMotorTargetSpeed) {
						MOTOR_LF = (unsigned char)speed;
					}
					else {
						MOTOR_LF = leftMotorTargetSpeed;
					}
				}
				else {
					speed = MOTOR_LF - leftMotorAccel;
					if (speed < 0 ) {
						MOTOR_LF = 0;
					}
					else {
						if (speed >= leftMotorTargetSpeed) {
							MOTOR_LF = (unsigned char)speed;
						}
						else {
							MOTOR_LF = leftMotorTargetSpeed;
						}
					}
				}
			} // endif (MOTOR_LF . . .)
			
			if (MOTOR_RT != rightMotorTargetSpeed) {		// Adjust right motor if necessary
				if (MOTOR_RT < rightMotorTargetSpeed) {
					speed = MOTOR_RT + rightMotorAccel;
					if (speed <= rightMotorTargetSpeed) {
						MOTOR_RT = (unsigned char)speed;
					}
					else {
						MOTOR_RT = rightMotorTargetSpeed;
					}
				}
				else {
					speed = MOTOR_RT - rightMotorAccel;
					if (speed < 0 ) {
						MOTOR_RT = 0;
					}
					else {
						if (speed >= rightMotorTargetSpeed) {
							MOTOR_RT = (unsigned char)speed;
						}
						else {
							MOTOR_RT = rightMotorTargetSpeed;
						}
					}
				}
			} // endif (MOTOR_RT . . .)
		} // endif (flags.motorRampEnabled)

		//	ALARM LED.
		if (flags.enableAlarm) {
			alarm--;
			if (alarm == 0) {
				ALARM_LED ^= 1;			// Toggle LED2
				alarm = ALARM_RATE;	// Reset "alarm"
			}
		}
		
		//	HEARTBEAT LED.
		if (flags.enableHeartbeat) {
			heartbeat--;
			if (heartbeat == 0) {
				HEARTBEAT_LED ^= 1;			// Toggle LED1
				heartbeat = HEARTBEAT_RATE;	// Reset "heartbeat"
			}
		}

	}
} // End of LowPriorityTable()

//***************************************************************************
//	High priority interrupts ADC
//***************************************************************************
#pragma interrupt HighPriorityTable
void HighPriorityTable(void)
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//	High priority interrupts
//		ADC
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
{
	if (PIR1bits.ADIF == 1) {
		PIR1bits.ADIF = 0;					// Clear the interrupt flag
		ADCresults[chnlIndex] = ADRESH;		// Put conversion result into ADCresults[]
		chnlIndex++;						// Point to next channel to be converted
		if (chnlIndex > 1) {
			chnlIndex = 0;					// Wrap if chnlIndex has overflowed
		}
		ADCON0bits.CHS = 0b00000;			// Clear channel field
		ADCON0 = ADCON0 | (lineChnls[chnlIndex] << 2);	// Set channel field with the next channel to be converted
		ADCON0bits.GO = 1;					// Start conversion on the next channel
	}
} // End of HighPriorityTable()
