// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//	Header file for Main.c
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//
//***************************************************************************
//								PIC18F2550
//						    +------------------+
//			  	  RESET --> | 1 MCLR	 RB7 28| <-- IR_LEFT
//			 	  LineL --> | 2 RA0/AN0	 RB6 27| <-- BTN
//				  LineR --> | 3 RA1/AN1	 RB5 26| <-- IR_RIGHT
//				        --> | 4 RA2/AN2	 RB4 25| --> IR_CTRL 
//				        --> | 5 RA3/AN3	 RB3 24| --> MOTB_DIR1
//	 		            --> | 6 RA4		 RB2 23| --> MOTB_DIR2
//	   	 		        --> | 7 RA5		 RB1 22| --> MOTA_DIR2
//					GND	--> | 8 VSS		 RB0 21| --> MOTA_DIR1
//	 			   OSC1 <-- | 9 		 VDD 20| <-- 5 volts
//	 			   OSC2 <-- |10 		 VSS 19| <-- GND
//		  HEARTBEAT_LED <-- |11 RC0		 RC7 18| <-- UART:RX
//		       MOTA_PWM <-- |12 CCP2	 RC6 17| --> UART:TX
//		       MOTB_PWM <-- |13 CCP1	 RC5 16| --> D+
//				   Vusb <-- |14 Vusb	 RC4 15| --> D-
//						    +------------------+
//
//***************************************************************************
//
//  Configuration bits
#pragma config PLLDIV   = 5         // (20 MHz crystal)
#pragma config CPUDIV   = OSC1_PLL2 // use PLL for clock
#pragma config USBDIV   = 2         // Clock source from 96MHz PLL/2
#pragma config FOSC     = HSPLL_HS  // high-speed crystal, LLL enabled
#pragma config FCMEN    = OFF       //
#pragma config IESO     = OFF       // 
#pragma config PWRT     = OFF       // power-up timer disabled
#pragma config BOR      = OFF       // brown-out reset disabled
#pragma config VREGEN   = ON        // USB Voltage Regulator enabled
#pragma config WDT      = OFF       // watchdog timer not used
#pragma config WDTPS    = 32768     // long watchdog value - not used anyhow
#pragma config MCLRE    = ON        // MCLR pin used as reset
#pragma config LPT1OSC  = OFF
#pragma config PBADEN   = OFF       // port B is digital by default
#pragma config STVREN   = ON
#pragma config LVP      = OFF       // no low voltage programming
#pragma config XINST    = OFF       // Extended Instruction Set disabled
#pragma config CCP2MX   = ON        // if CCP2 is used, leave it on the port C pin
#pragma config CP0      = OFF       // code protect disabled
#pragma config CP1      = OFF       // code protect disabled
#pragma config CPB      = OFF       // code protect disabled
#pragma config WRT0     = OFF
#pragma config WRT1     = OFF
#pragma config WRTB     = OFF       // Boot Block Write Protection off
#pragma config WRTC     = OFF
#pragma config EBTR0    = OFF
#pragma config EBTR1    = OFF
#pragma config EBTRB    = OFF
//  *** Booleans ***
#define  TRUE  1
#define  FALSE  0
// handy things to make the code more readable
#define BOTH      0
#define LEFT      1
#define RIGHT     2
#define BACK      3
#define FWD       0
#define FORWARD   0
#define BWD       1
#define REVERSE   1
#define STOP      2
#define LED_OFF   0
#define LED_ON    1
// LEDs 
#define LED1           LATCbits.LATC0  // user LED1 in the schematic
#define LED2           LATAbits.LATA4  // user LED2 in the schematic
#define HEARTBEAT_LED  LATCbits.LATC0  // heartbeat on LED1 if user enables
#define ALARM_LED      LATAbits.LATA4  // alarm condition on LED2 if user enables 
#define HEARTBEAT_RATE 1000            // heartbeat blink rate in milliseconds
#define ALARM_RATE     200             // alarm blink rate in milliseconds
// Button
#define	BTN		  PORTBbits.RB6
// Enable IR Detectors
#define IR_ENABLE      LATBbits.LATB4  // Enable 555 timer to generate 38khz for IR Enemy Detection 
// Motors
#define	MOTOR_LF		CCPR2L
#define	MOTOR_RT		CCPR1L
#define	MOTB_DIR2		LATBbits.LATB3 // inp1 Right Motor
#define	MOTB_DIR1		LATBbits.LATB2 // inp2 Right Motor
#define	MOTA_DIR2		LATBbits.LATB1 // inp4 Left Motor
#define	MOTA_DIR1		LATBbits.LATB0 // inp3 Left Motor
#define	BOTH			0
#define	LEFT			1
#define	RIGHT			2
#define	FWD				0
#define	BWD				1
// Infrared
#define  IR_LEFT  PORTBbits.RB7 // input for detecting IR activity on IR#1 (left)
#define  IR_RIGHT PORTBbits.RB5 // input for detecting IR activity on IR#2 (right)
//Convenient names for the line detector values in the ADC array
#define  LineL  ADCresults[0] // Left Line
#define  LineR   ADCresults[1] // Right Line
#define ADC_MIN  0  // readings below this turn are turned to 0x00
#define LINE_THRESHOLD 100 
// Speed
#define  SPEED_FULL 	255 
#define  SPEED_MED      200 
#define  SPEED_OFF      0 
// Hunt counters
#define  COUNTER_SPIN_COUNT      15    // 0 - 14 = hunt arc, 15 - 34 = hunt spin
#define  COUNTER_TURN_COUNT      35	   // 35 - 54 = hunt straight 
#define  COUNTER_STRAIGHT_COUNT  55    // 55 = wrap hunt couner
#define  HUNT_LEFT            1
#define  HUNT_RIGHT           2
#define  SWAP_HUNT_DIR(dir)   (3 - (dir))
#define  HUNT_STATE_SPIN_LEFT    1
#define  HUNT_STATE_SPIN_RIGHT   2
#define  HUNT_STATE_GO_STRAIGHT  3
#define  COUNTER_EYE_RESET_COUNT ( COUNTER_SPIN_COUNT + 5 )
// IR Object detection
#define  RANGE_MIN_VALUE 50
#define  RANGE_MAX_VALUE 60
// Line detection
#define  LINE_DETECTED_LEFT   0x01
#define  LINE_DETECTED_RIGHT  0x02
#define  EYE_DETECTED_LEFT    0x01
#define  EYE_DETECTED_RIGHT   0x02
// UART
#define  UART_TX  LATCbits.LATC6 // UART transmit pin
#define  UART_RX  PORTCbits.RC7  // UART receive pin
// Handy UART BAUD rates : Fosc = 48 MHz, Fcy = Fosc/4 = 12 MHz
#define  BAUD_230400     12    // SPBRG = ( (Fosc/230400) / 16 ) - 1 = 12
#define  BAUD_115200     25
#define  BAUD_56700      52
#define  BAUD_38400      77
#define  BAUD_19200     155

