// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//	Motor functions
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

//	*** Variables ***
extern unsigned char 	leftMotorTargetSpeed, rightMotorTargetSpeed;	// Target speeds 
extern unsigned char 	leftMotorAccel, rightMotorAccel;				// Allowed accelerations
	
void disableMotors(void)
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//	disableMotors
//		Puts the motor drivers into the high impedance state, ie. OFF
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
{
	MOTA_DIR1 = 0; MOTA_DIR2 = 0;	// Disable the left motor
	MOTB_DIR1 = 0; MOTB_DIR2 = 0;	// Disable the right motor
	flags.motorRampEnabled = FALSE;	// Speed ramping is not allowed.
    leftMotorTargetSpeed = 0;		// Target speed is stopped.
    rightMotorTargetSpeed = 0;		// Target speed is stopped.
} // End of disableMotors()



void setMotorDirection(unsigned char whichMotor, unsigned char direction)
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//	setMotorDirection
//		whichMotor		= BOTH, LEFT or RIGHT
//		direction		= FWD or BWD
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
{
	switch (whichMotor) {
		case BOTH:
			switch (direction) {
				case FWD:
					MOTA_DIR1 = 1;
					MOTA_DIR2 = 0;
					MOTB_DIR1 = 1;
					MOTB_DIR2 = 0;
				break;
				
				case BWD:
					MOTA_DIR1 = 0;
					MOTA_DIR2 = 1;
					MOTB_DIR1 = 0;
					MOTB_DIR2 = 1;
				break;
			}
		break;
		
		case LEFT:
			switch (direction) {
				case FWD:
					MOTA_DIR1 = 1;
					MOTA_DIR2 = 0;
				break;
				
				case BWD:
					MOTA_DIR1 = 0;
					MOTA_DIR2 = 1;
				break;
			}
		break;
		
		case RIGHT:
			switch (direction) {
				case FWD:
					MOTB_DIR1 = 1;
					MOTB_DIR2 = 0;
				break;
				
				case BWD:
					MOTB_DIR1 = 0;
					MOTB_DIR2 = 1;
				break;
			}
		break;
	}
} // End of setMotorDirection()

void motorControl(unsigned char whichMotor, unsigned char direction, unsigned char speed, signed char acceleration)
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//	motorControl
//		whichMotor		= BOTH, LEFT or RIGHT
//		direction		= FWD or BWD
//		speed			= 0..255
//		acceleration	= speed increment/decrement allowed per 1 ms period
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
{
	switch (whichMotor) {
		case BOTH:
			switch (direction) {
				case FWD:
					MOTA_DIR1 = 1;
					MOTA_DIR2 = 0;
					MOTB_DIR1 = 1;
					MOTB_DIR2 = 0;
				break;
				
				case BWD:
					MOTA_DIR1 = 0;
					MOTA_DIR2 = 1;
					MOTB_DIR1 = 0;
					MOTB_DIR2 = 1;
				break;
			}
			MOTOR_LF = speed;
			MOTOR_RT = speed;
			leftMotorAccel = acceleration;
			rightMotorAccel = acceleration;
		break;
		
		case LEFT:
			switch (direction) {
				case FWD:
					MOTA_DIR1 = 1;
					MOTA_DIR2 = 0;
				break;
				
				case BWD:
					MOTA_DIR1 = 0;
					MOTA_DIR2 = 1;
				break;
			}
			MOTOR_LF = speed;
			leftMotorAccel = acceleration;
		break;
		
		case RIGHT:
			switch (direction) {
				case FWD:
					MOTB_DIR1 = 1;
					MOTB_DIR2 = 0;
				break;
				
				case BWD:
					MOTB_DIR1 = 0;
					MOTB_DIR2 = 1;
				break;
			}
			MOTOR_RT = speed;
			rightMotorAccel = acceleration;
		break;
	}
} // End of motorControl()

