//***************************************************************************
//   smashndash.c
//
//   This module is the main definition for Smash 'n Dash, an
//   autonomous mini-sumo robot.
//***************************************************************************

unsigned char EyeMinSignal;
unsigned char EyeState;
unsigned char PrevEyeState;
unsigned char HuntDir;
unsigned int  HuntCounter;
unsigned char DirectionL;
unsigned char DirectionR;
unsigned char lineSide;
unsigned char currentState;

enum mainStates {
    WAITING = 0,
    COUNTDOWN = 1,
    HUNT = 2,
    ATTACK = 3,
    SURVIVE = 4
};

void fsmInitialize()
{
    currentState = WAITING;
}

void handleWaiting()
{
    unsigned int i = 0;
    checkEdges = FALSE;
    disableMotors();
    while (BTN == 1) {
	delay_ms(50);
    }
    while (BTN == 0) {
	delay_ms(100);
	currentState = COUNTDOWN;
    }
}

void handleCountdown()
{
    unsigned char i = 0;

    // count down 5 seconds
    for (i = 5; i > 0; i--) {
	delay_ms(1000);
    }

    disableMotors();
    HuntCounter = 0;
    EyeState = 0;
    PrevEyeState = 0;
    EyeMinSignal = RANGE_MIN_VALUE;
    HuntDir = HUNT_LEFT;
    currentState = HUNT;
}

void handleHunt()
{
    checkEdges = TRUE;
    HuntCounter++;

    if ((LineL > LINE_THRESHOLD) || (LineR > LINE_THRESHOLD))
	currentState = SURVIVE;
    else if (leftContact || rightContact)
	currentState = ATTACK;

    // No object detected, enter hunt mode.

    if (HuntCounter < COUNTER_TURN_COUNT) {

	if (HuntDir == HUNT_LEFT) {

	    // Hunt State - Spin Left

	    if (HuntCounter < COUNTER_SPIN_COUNT) {
		// Arc left 
		leftMotorTargetSpeed = SPEED_MED;
		DirectionL = FWD;
	    } else {
		// Spin left 
		leftMotorTargetSpeed = SPEED_MED;
		DirectionL = BWD;
	    }

	    rightMotorTargetSpeed = SPEED_MED;
	    DirectionR = FWD;
	} else {
	    // Hunt State - Spin Right 
	    leftMotorTargetSpeed = SPEED_MED;
	    DirectionL = FWD;

	    if (HuntCounter < COUNTER_SPIN_COUNT) {
		// Arc right 
		rightMotorTargetSpeed = SPEED_MED;
		DirectionR = FWD;
	    } else {
		// Spin right 
		rightMotorTargetSpeed = SPEED_MED;
		DirectionR = BWD;
	    }
	}
    } else if (HuntCounter < COUNTER_STRAIGHT_COUNT) {
	// Hunt State - Go Straight
	leftMotorTargetSpeed = SPEED_MED;
	DirectionL = FWD;
	rightMotorTargetSpeed = SPEED_MED;
	DirectionR = FWD;
    } else {
	// Counter wrapped and we still didn't find anybody.
	// Swap hunt direction so we don't get "stuck in a corner"
	// Swap hunt direction 
	HuntCounter = 0;
	HuntDir = SWAP_HUNT_DIR(HuntDir);
    }

    if (HuntCounter >= COUNTER_EYE_RESET_COUNT) {
	EyeMinSignal = RANGE_MIN_VALUE;
    }

    flags.motorRampEnabled = FALSE;
    motorControl(LEFT, DirectionL, leftMotorTargetSpeed, 1);
    motorControl(RIGHT, DirectionR, rightMotorTargetSpeed, 1);
}

void handleAttack()
{
    checkEdges = TRUE;
    EyeState = 0;


    if ((LineL > LINE_THRESHOLD) || (LineR > LINE_THRESHOLD)) {
	currentState = SURVIVE;
    }

    else if (ObjectStraightAhead) {
	// Attack - Forward
	flags.motorRampEnabled = FALSE;
	motorControl(BOTH, FWD, SPEED_FULL, 1);
    } else if (leftContact) {
	// Object detected on the left side only. Turn towards it
	EyeState = EYE_DETECTED_LEFT;

	if (HuntDir != HUNT_LEFT) {
	// Reset HuntCounter 
	    HuntCounter = 0;
	}

	HuntDir = HUNT_LEFT;
	// Attack - Left 
	flags.motorRampEnabled = FALSE;
	motorControl(LEFT, FWD, SPEED_OFF, 1);
	motorControl(RIGHT, FWD, SPEED_MED, 1);
    } else if (rightContact) {
	// Object detected on the right side only. Turn towards it
	EyeState = EYE_DETECTED_RIGHT;

	if (HuntDir != HUNT_RIGHT) {
	// Reset HuntCounter 
	    HuntCounter = 0;
	}

	HuntDir = HUNT_RIGHT;
	// Attack - Right 
	flags.motorRampEnabled = FALSE;
	motorControl(LEFT, FWD, SPEED_MED, 1);
	motorControl(RIGHT, FWD, SPEED_OFF, 1);
    } else {
	checkEdges = TRUE;
	// Reset HuntCounter 
	HuntCounter = 0;
	currentState = HUNT;
    }
}

void handleSurvive()
{
    // Line detected, survive mode

    lineSide = 0;

    if (LineR > LINE_THRESHOLD) {
	// Line Detected - Right
	lineSide |= LINE_DETECTED_RIGHT;

	if (!ObjectStraightAhead) {
		// Set the hunt direction to be left so that we hunt "away" from the line.
	    HuntDir = HUNT_LEFT;
	}
    }
    if (LineL > LINE_THRESHOLD) {
	// Line Detected - Left              
	lineSide |= LINE_DETECTED_LEFT;
	if (!ObjectStraightAhead) {
		// Set the hunt direction to be right so that we hunt "away" from the line.
	    HuntDir = HUNT_RIGHT;
	}
    }

    disableMotors();
    delay_ms(5);
    flags.motorRampEnabled = FALSE;
    leftMotorTargetSpeed = SPEED_MED;
    DirectionL = BWD;
    rightMotorTargetSpeed = SPEED_MED;
    DirectionR = BWD;
    motorControl(LEFT, DirectionL, leftMotorTargetSpeed, 1);
    motorControl(RIGHT, DirectionR, rightMotorTargetSpeed, 1);
    delay_ms(100);
	
    // Turn away from the line

    if ((lineSide & LINE_DETECTED_LEFT) != 0) {
	leftMotorTargetSpeed = SPEED_MED;
	DirectionL = FWD;
    } else {
	rightMotorTargetSpeed = SPEED_MED;
	DirectionR = FWD;
    }
    flags.motorRampEnabled = FALSE;
    motorControl(LEFT, DirectionL, leftMotorTargetSpeed, 1);
    motorControl(RIGHT, DirectionR, rightMotorTargetSpeed, 1);
    delay_ms(80);

    // Raise the eye threshold since we hit the line
    // Gets a bit annoying when it tries to chase the couch
    EyeMinSignal = 78;

    // Set the Hunt Counter to spin
	
    HuntCounter = COUNTER_SPIN_COUNT;
    currentState = HUNT;

}

void fsmExecute()
{
    switch (currentState) {

    case WAITING:
	handleWaiting();
	break;
    case COUNTDOWN:
	handleCountdown();
	break;
    case HUNT:
	handleHunt();
	break;
    case ATTACK:
	handleAttack();
	break;
    case SURVIVE:
	handleSurvive();
	break;
    }
}
